/* Have tested with:
 *   Arduino IDE         v1.8.8
 *   Adafruit SSD 1306   v1.2.9  must comment out the drawBitmap calls in Adafruit_SSD1306.cpp to save enough memory to compile
 *    
  if (HEIGHT > 32) {
    drawBitmap((WIDTH - splash1_width) / 2, (HEIGHT - splash1_height) / 2,
               splash1_data, splash1_width, splash1_height, 1);
  } else {
    drawBitmap((WIDTH - splash2_width) / 2, (HEIGHT - splash2_height) / 2,
              splash2_data, splash2_width, splash2_height, 1);
  }

 *   Adafruit GFX        v1.4.7 
 *   Adafruit Neopixel   v1.1.8 
 *   RTC by Makuna       v2.3.2
 *   
 *   Compiled size: 29712 96% program storage space, 546 26% dynamic memory
*/


/* Misc notes:
 *   SSD 1306 lib uses a big chunk of dynamic memory, so have a lot less to work with than compiler says is available,
 *   program starts acting eraticly if using over 39% or so.
 *   
 *   Not using getTextBounds because it uses at least 1k of flash mem and sometimes it was returning bad values 
 *   
 *   To save about 1.5k of flash memory comment out the if/else section 
 *   with the drawBitmap calls in the constructor in Adafruit_SSD1306.cpp
 *    
  if (HEIGHT > 32) {
    drawBitmap((WIDTH - splash1_width) / 2, (HEIGHT - splash1_height) / 2,
               splash1_data, splash1_width, splash1_height, 1);
  } else {
    drawBitmap((WIDTH - splash2_width) / 2, (HEIGHT - splash2_height) / 2,
              splash2_data, splash2_width, splash2_height, 1);
  }

 *   
 */

 /* Brett Oliver's notes http://www.brettoliver.org.uk/      http://brettoliver.org.uk/Ball_Bearing_Clock/Ball_Bearing_Clock.htm
  *  adding 30 sec sync
  *  v03 ball detect changed to IR optical sensor changed input to 10K pull down resistor RISING not FALLING
  *  v04 z1_loopFunctions 206 enable ball detect after 1s to stop false triggers when motor starts added Secondtimer so motor starting does not false trigger ball detect
         also number of LEDs changed see includes & init line 86  - not used
     Note     30 second sync to locate seach for //30 second sync
         30 second sync code locations - The TimeVortex-Sketch06 line 45   z1_LoopFunctions line 10  z3_RTCStuff line 85
     v08 Brightness added
     v09 rainbowMinDelay changed my globals 9
     v10 rainbowMinDelay changed my globals 6
     v11 adding 30 sec LEDs in control box
     v12 changed push top button to push select z2 writescreen buffer 86
     v13 added LED to power switch bracket and Ball Bearing Clock Sign
     v14 added 30 sec sync to +ve pulse on D3 as well as -ve pulse D4
     v15 clock will not sync in "QUIET" mode to prevent loss of re-syncing ball time/in myglobals const uint16_t ISRDetachMillis = 1000; set to 1000
         also changed 30 sec sync to 25 to 35 seconds only if (now.Second() >= 25 && now.Second() <= 35 )
     v16 Added PIR to control LEDs AND oled DISPLAY added secsReady to allow sync at certain seconds only
  */


//At 29238 95%,  542 26% now with v1.2.9
//   29210 95%, 1566 76%     with v1.1.2 

//Now at 29484 95%, 545 26%



#include "includesAndInit.h"
#include "myGlobals.h"

void loop()
{  




 
 // secsync becomes 1 at 5 seconds see z1 loop function line 11
 if (digitalRead(secsyncPin) == LOW && secsync == 1 && secsReady == 1 )
{
    
   resetSeconds30();
   secsync = 0;
   
   
}
//Synch seconds to 30 from master clock
if (digitalRead(secPossyncPin) == HIGH && secsync == 1 && inQuietModeNow == false && secsReady == 1)// inQuietModeNow == false sync only happens when QUIET mode is off
{
    
   resetSeconds30();
   secsync = 0;
   
   
}

// Display synce pulse on control panel
// secsync allways becomes 1 at 10 seconds
// second sync only becomes 0 on reciept of 30second sync pulse from a master clock or if 30 sec sync button is pressed
// once a 30 second sync pulse is recieved another 30 second sync pulse can not be recieved until after 35 seconds have elasped (5 secs past the next min)



if (secsync == 1 )
{
   
   SecSyncLEDColor=purple;
   SecSyncLEDColora=purple;
}
if ( secsync == 0 )
{
   
   SecSyncLEDColor=green;
   SecSyncLEDColora=green;
}


//PIR
if (digitalRead(PIRPin) == LOW && PIROn == 1 )
{
   display.ssd1306_command(SSD1306_DISPLAYOFF);
   strip.setBrightness(0);
   strip.show();
  
   PIROn = 0; // allows PIR to triger once
}

if (digitalRead(PIRPin) == HIGH && PIROn == 0 )
{
 
 display.ssd1306_command(SSD1306_DISPLAYON);
 strip.setBrightness(Brightness);
 strip.show();
 PIROn = 1; //allows PIR to triger once
 
}

 
 //Brightness  
 if (digitalRead(button3Pin) == LOW && secondcheck != masterTime.Second())
{
   Brightness += 20;
   strip.setBrightness(Brightness);   
   
   
   if (Brightness > 240)
   {
    Brightness = 10;
    strip.setBrightness(Brightness);
   }
  // Serial.print("Brightness ");
  //  Serial.println(Brightness);
  strip.show();
 secondcheck = masterTime.Second();
}


 
  
  updateMasterTimeFromRTC();
  checkIfNeedToBeQuietOrWakeUp();
  handleSignRainbowMode();
 
  if (ballDeliveryError==false && inQuietModeNow==false)
  {
    handleMotorStateForNormalAndDemoModes();  
    serviceBallDetectISR();
    motorOnOrOff();
    updateFrontPanelLED();
    updateDisplayBufferAndHandleButtonActions();    
    writeDirtyDisplayBuffer();
    writeEEPROMIfNeeded();
  }
  else if (isButtonPressed(&button1History) || isButtonPressed(&button2History))
  {
    if (ballDeliveryError)
    {
      ballDeliveryError=false;
      setMasterTimeFromRtcAndSyncIfNeeded();
    }

    if (inQuietModeNow)
    { 
      endQuietTimeMode();
      quietTimeEndOverride=true;
    }
    
    forceDisplayUpdate=true;
  }

  updateNeoPixelsIfNeeded();

  secondChanged=false;
  minuteChanged=false;
}


//************************************  Misc functions *****************************************************************************************


void ISR_ballDetect()
{
  ballDetectISRCount++;
}


void turnMotorOff()
{
  analogWrite(motorPin, 0);
  liftTubeLEDColor=off;
  motorOn=false;
}


void turnMotorOn()
{
  analogWrite(motorPin, motorPWM);
  motorTurnedOnMillis=millis();
  liftTubeLEDColor=blue;
  motorOn=true;
}


void incrementDisplayState()
{
  byte state=displayState;
  state++;
  displayState=state;
  if (displayState>=LAST)
    displayState=TIME_OR_DEMO;

  forceDisplayUpdate=true;
}


void changeMode()
{ 
  incByte(mode, 3);
  if (mode==0)
  {
    turnMotorOff();
    motorState=0;
    initializeBallsDeliveredToRTC();
  }
  forceDisplayUpdate=true;
}


void incByte(byte &val, const byte endVal)
{
  val++;
  if (val>endVal)
    val=0;
}


boolean isItQuietTimeNow()
{
  uint16_t nowTotalMinutes=masterTime.Hour()*60+masterTime.Minute();
  if (persistentData.quietTimeEnd < persistentData.quietTimeStart) //Then quiet time crosses midnight boundry
      {
        if ((nowTotalMinutes >= persistentData.quietTimeStart) || (nowTotalMinutes < persistentData.quietTimeEnd))
          return true;
      }
      else //Doesn't cross midnight
      {
        if ((nowTotalMinutes >= persistentData.quietTimeStart) && (nowTotalMinutes < persistentData.quietTimeEnd))
          return true;
      }

      return false;

}

/*
int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}
*/
