//Misc constants
const byte motorPWM = 255*1.0; //was .70              //Adjust PWM so ball delivery time is at least 1 second, otherwise will get balls dropped in continuous mode
const byte ballDeliveryTimeout = 60*2;       //Number of seconds to wait for ball delivery before timeout
const byte minutesToWaitForTimeToCatchup=25; //If need to sync for instance 11 hours and 50 minutes it will be more efficient to wait 10 minutes for time to catch up instead of delivering 710 balls 
const uint16_t ISRDetachMillis = 1000;        //Number of milliseconds to detach the ball detect ISR before reattaching
const uint16_t statsScreenUpdateMillis=250;  //Number of milliseconds to wait for each stats screen update
const uint16_t buttonHoldMinimumMillis=500;
const uint16_t buttonHoldRepeatMillis=150;
const byte rainbowMinDelay = 6;//10             //Miniumum number of milliseconds to wait before changing LED's in sign in rainbow mode
const byte neoPixelNormalBrightness = 30;
const byte neoPixelQuietBrightness = 10;
#ifdef MORE_SIGN_MODES
  const byte maxSignModes = 8;
#else
  const byte maxSignModes = 4;
#endif


//RTC, internal timer stuff                
RtcDateTime masterTime=0;                   //The main thing to keep track of
boolean secondChanged=false;
boolean minuteChanged=false;
byte minutesLeftToWait=0;
unsigned long baseMillis=0;                 //Just curious to see the drift of the internal RTC to the external
unsigned long millisOffset=0;               //
unsigned long ballsDelivered=0;             //Keeps track of the number of balls delivered in normal mode, used to sync on power restore
uint16_t rtcBadReads=0;

//Mode (Normal/Demo) stuff
byte mode=0;                                //For demo mode, 0 normal op, 1 demo mode 0, 2 demo mode 1, etc.
int demoCountdownTimer=0;                   //In demo mode the count down time before motor is turned back on

//Motor stuff
int motorState=0;                           //If 1, turn on motor, can be greater than 1 if ball delivery is behind schedule
boolean motorOn=false;
unsigned long motorTurnedOnMillis=0;        //Last timer tick motor was turned on

//Ball stuff
boolean ballDeliveryError=false;
boolean ballDetectISRAttached=true;
unsigned long lastBallDetectMillis=0;       //Last timer tick a ball was detected
volatile int ballDetectISRCount=0;          //Number of times ball switch ISR is triggered per run
unsigned long lastBallDeliveryMillis=0;     //For stats
//unsigned long ballCount=0;                  //For stats
//unsigned long totalBallDeliveryMillis=0;    //For stats
//float lastBallDeliveryTime=0;               //For stats
//float minBallDeliveryTime=__LONG_MAX__;     //For stats
//float maxBallDeliveryTime=0;                //For stats

//Display stuff
boolean displayUpdated=false;
boolean forceDisplayUpdate=true;
enum displayStates{TIME_OR_DEMO, CHANGE_MODE, QUIET_SETUP, QUIET_NOW, RESET_SECONDS, STATS1, LAST, SYNC, WAIT, MOD_QUIET_SETTINGS};
displayStates displayState=TIME_OR_DEMO;

//Quiet hours stuff
uint8_t interQuietTimeStartHour=0;
uint8_t interQuietTimeStartMinute=0;
uint8_t interQuietTimeEndHour=0;
uint8_t interQuietTimeEndMinute=0;
uint8_t interCurrentTimeHour=0;
uint8_t interCurrentTimeMinute=0;
boolean currentTimeChanged=false;
uint8_t interQuietTimeEnabled=0;
uint8_t quietSetupFieldPtr=0;
boolean inQuietModeNow=false;
boolean quietTimeStartOverride=false;
boolean quietTimeEndOverride=false;


//Persistent data is written to the eeprom when it's changed and read from the eeprom on startup
struct persistentStruct
{
  byte signMode;                            //0=Off, 1=Rainbow, 2=Blue, 3=Green, 4=Cyan, 5=Yellow, 6=Purple, 7=Red, 8=White  
  uint16_t quietTimeStart;
  uint16_t quietTimeEnd;
  boolean quietTimeEnabled;
  uint16_t crc;
};
persistentStruct persistentData;
boolean persistentDataChanged=false;


//Neopixel stuff
boolean neoPixelUpdated=true;
uint32_t panelLEDColor=0;
uint32_t liftTubeLEDColor=0;
uint32_t SecSyncLEDColor=0;
uint32_t SecSyncLEDColora=0;
uint32_t PowerLEDColor=0;

//Button stuff
volatile uint8_t button1History=0;
volatile uint8_t button2History=0;
