/*
void getTextWidthAndHeight(char *str, byte *w, byte *h)
{
  int16_t  x1, y1;
  display.getTextBounds(str, 0, 63, &x1, &y1, (uint16_t)w, (uint16_t)h);
  debugPrint(F("str width="));
  debugPrintln(*w);
}
*/
   

//**************************  Helpers ********************************************************************************

void clearDisplay()
{
  displayUpdated=true;
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setFont();
  display.setTextSize(1);
  display.setCursor(0, 0);
}


void displayValueWithLeadingZero(byte val)
{
  if (val<10)
    display.print('0');
  display.print(val);
}


void displayDate(const __FlashStringHelper *desc, byte hours, byte minutes, byte seconds)
{
  display.print(desc);
  display.print(hours);
  display.print(':');
  displayValueWithLeadingZero(minutes);
  display.print(':');
  displayValueWithLeadingZero(seconds);
}


void invertFieldIfMatch(const byte matchNum)
{
  if (quietSetupFieldPtr==matchNum)
    display.setTextColor(BLACK, WHITE);
  else
    display.setTextColor(WHITE);
}


void displayStrAndValueInvertMatch(const __FlashStringHelper *str, byte val, const byte matchNum)
{   
  display.print(str);
  invertFieldIfMatch(matchNum);
  displayValueWithLeadingZero(val);
  display.setTextColor(WHITE);
}


void writeTitleScreenToBuffer(const __FlashStringHelper *title, const byte w)
{
  const byte h=11;
  const byte xOff=(127-w)/2;
  const byte yOff=h;
    
  clearDisplay(); 
  display.setFont(MENUFONT);
  display.setCursor(xOff, yOff);
  display.println(title);
  display.drawLine(xOff, yOff+4, xOff+w, yOff+4, WHITE);
  display.setCursor(0, yOff*3.5);
}


//**************************  Main screens ********************************************************************************


void writeTitleActionScreenToBuffer(const __FlashStringHelper *title, const byte w, const __FlashStringHelper *action)
{  
  if (forceDisplayUpdate)
  {
    writeTitleScreenToBuffer(title, w);
    
    display.print(F("Push select\nto "));
    display.println(action);
    forceDisplayUpdate=false;
  }
}


void writeChangeModeScreenToBuffer()
{
  if (forceDisplayUpdate)
  { 
    writeTitleScreenToBuffer(F("Change Mode:"), 90);
    
    display.println(F("Mode:"));
    if (mode==0)
      display.println(F("Normal"));
    else if (mode==1)
      display.println(F("Continuous demo"));
    else
    {
      display.print(F("Demo "));
      display.print(mode-1);
      display.println(F(" sec delay"));
    }
    forceDisplayUpdate=false;
  }      
}


void writeModQuietSettingsScreenToBuffer()
{
  if (minuteChanged)
  {
    interCurrentTimeHour=masterTime.Hour();
    interCurrentTimeMinute=masterTime.Minute();
    forceDisplayUpdate=true;
  }
  
  if (forceDisplayUpdate)
  {
    clearDisplay();

    displayStrAndValueInvertMatch(F("Current time:"), interCurrentTimeHour, 0);
    displayStrAndValueInvertMatch(F(":"), interCurrentTimeMinute, 1);
    
    display.print(F("\n\nQuiet time: "));
    invertFieldIfMatch(2);
    if (interQuietTimeEnabled)
      display.println(F("ENABLED"));
    else
      display.println(F("DISABLED"));
    display.setTextColor(WHITE);
    displayStrAndValueInvertMatch(F("Start time:  "), interQuietTimeStartHour, 3);
    displayStrAndValueInvertMatch(F(":"), interQuietTimeStartMinute, 4);
    displayStrAndValueInvertMatch(F("\nEnd time:    "), interQuietTimeEndHour, 5);
    displayStrAndValueInvertMatch(F(":"), interQuietTimeEndMinute, 6);
    
    invertFieldIfMatch(7);  
    display.println(F("\n\nSave"));
    invertFieldIfMatch(8); 
    display.println(F("Exit"));
    
    forceDisplayUpdate=false;
  }
}


void writeSecondsOrDemoScreenToBuffer()
{      
  static unsigned long animateMoveMillis=0;
  unsigned long currentMillis=millis();
  if (mode>0 && motorState>0 && currentMillis>animateMoveMillis)
    forceDisplayUpdate=true;

  if (forceDisplayUpdate)
  {
    clearDisplay();
      
    if (mode==0)
    {   
      //Keep the space between the 2 digits centered on the x axis
      const byte xPos=63;
      const byte yPos=52;
      static const byte leftDigitOffset[]={27, 19, 26, 27, 27, 27};
      static const byte rightDigitOffset[]={0, 2, 0, 0, 2, 0, 0, 2, 0, 0};
      
      byte leftDigit=masterTime.Second()/10;
      byte rightDigit=masterTime.Second()%10;

      display.setFont(&impact27pt7b);
      display.setCursor(xPos-leftDigitOffset[leftDigit], yPos);
      display.print(leftDigit);
      display.setCursor(xPos+1+rightDigitOffset[rightDigit], yPos);
      display.print(rightDigit);
      //display.drawLine(0, 9, 127, 9, WHITE);
      //display.drawLine(0, 54, 127, 54, WHITE);
    }
    else //In one of the demo modes
    {
      #ifndef ENABLE_SERIAL_DEBUG
        //Need more space if using serial to debug so don't load the font for demo mode
        display.setFont(&impact11pt7b);
      #endif
      display.setCursor(2, 16);
      display.print(F("Demo Mode: "));
      display.println(mode-1);

      if (mode==1)
      {
        display.setCursor(12, 39);  
        display.println(F("Continuous"));
      }
      else //mode > 1
      {
        //Display delta to 1/10 of a second, keep the decimal point centered on x axis
        const byte xPos=63;
        const byte yPos=41;
        static const byte leftDigitOffset[]={13, 9, 13, 13, 13, 13};
        static const byte rightDigitOffset[]={1, 2, 1, 1, 2, 1, 1, 2, 1, 1};
        
        byte leftDigit=demoCountdownTimer/1000;
        byte rightDigit=(demoCountdownTimer-(leftDigit*1000))/100;
        
        display.setFont(&impact14pt7b);
        display.setCursor(xPos-1-leftDigitOffset[leftDigit], yPos);
        display.print(leftDigit);
        display.setCursor(xPos-1, yPos);
        display.print('.');
        display.setCursor(xPos+2+rightDigitOffset[rightDigit], yPos);
        display.println(rightDigit);
        //display.drawLine(0, 20, 127, 20, WHITE);
        //display.drawLine(0, 43, 127, 43, WHITE);
      }  

      if (mode>0 && motorState>0 && currentMillis>animateMoveMillis)
      {
        const byte r=9;
        static byte dotPos=0;
      
        display.fillCircle(dotPos+r, 63-r, r, WHITE);
        dotPos+=r+r;
        if (dotPos+r >= 127)
          dotPos=0;  

        animateMoveMillis=currentMillis+500;
        forceDisplayUpdate=true;
      } 
    }

    forceDisplayUpdate=false;
  }
}


void writeStatsScreenToBuffer(byte screenNum)
{
  //Can display 8 lines with the built in font.
  
  unsigned long currentMillis=millis();
  static unsigned long lastUpdateMillis=0;
  if (currentMillis - lastUpdateMillis > statsScreenUpdateMillis)
  {
    forceDisplayUpdate=true;
    lastUpdateMillis=currentMillis;
  }
    
  if (forceDisplayUpdate)
  {
    clearDisplay();

    if (screenNum==1)
      stats1();
    //else
    //  stats2();
   
    forceDisplayUpdate=false;
  }
}


void stats1()
{
  unsigned long currentMillis=millis();
  //RtcDateTime now = RTC.GetDateTime();  //Master time isn't updated in demo mode

  //Time from external RTC
  displayDate(F("EXT: "), masterTime.Hour(), masterTime.Minute(), masterTime.Second());
    
  //Time from internal timer
  unsigned long val=(currentMillis+millisOffset-baseMillis)/1000;
  displayDate(F("\nINT: "), numberOfHours(val), numberOfMinutes(val), numberOfSeconds(val));

  display.print("\n\nBalldelvy time: ");
  display.println((float)lastBallDeliveryMillis/1000, 2);
  
  //display.print(F("Bad reads: "));
  //display.println(rtcBadReads);

  //display.print(F("Motor State: "));
  //display.println(motorState);

  /*
  byte pwmPercent=(byte)(((float)motorPWM/255.0)*100.0);
  display.print(F("PWM: "));
  if (pwmPercent < 100)
    display.print(F(" "));
  display.print(pwmPercent);
  display.println("%");
  */
}


/*
void stats2()
{
  display.print(F("BCount: "));
  display.println(ballCount);
  display.print(F("CurBDT: "));
  display.println(lastBallDeliveryTime, 2);
  display.print(F("MinBDT: "));
  display.println(minBallDeliveryTime, 2);
  display.print(F("MaxBDT: "));
  display.println(maxBallDeliveryTime, 2);
  display.print(F("AvgBDT: "));
  display.println((((float)totalBallDeliveryMillis/1000.0)/(float)ballCount), 2);  
}
*/


//**************************  Special mode screens ********************************************************************************


void writeSyncScreenToBuffer()
{
  if (forceDisplayUpdate)
  {
    clearDisplay();
  
    display.setFont(MENUFONT);
    display.setCursor(0, menuFontHeight);
    display.println(F("Synchronizing.\n"));
    display.print(F("Seconds="));
    display.println(masterTime.Second());
    display.print(F("Balls remaining:  "));
    display.println(motorState);
    forceDisplayUpdate=false;
  }
}

void writeWaitScreenToBuffer()
{
  if (forceDisplayUpdate)
  {
    clearDisplay();
    display.setFont(MENUFONT);
    display.setCursor(0, menuFontHeight);
    display.println(F("Waiting for time"));
    display.println(F("to catch up."));
    display.print(minutesLeftToWait);
    display.print(F(" minute"));
    if (minutesLeftToWait>1)
      display.print('s');  
    display.println(F(" left"));
    forceDisplayUpdate=false;
  }
}


void startBallDeliveryError()
{
  turnMotorOff();
  ballDeliveryError=true;
  
  clearDisplay();
  display.setFont(MENUFONT);
  display.setCursor(0, menuFontHeight);
  /*
  display.println(F("Ball delivery"));
  display.println(F("timeout."));
  display.println(F("Push button"));
  display.println(F("to continue..."));
  */
  display.println(F("Ball delivery\ntimeout.\nPush button\nto continue..."));
}


void startQuietTimeMode()
{
  turnMotorOff();
  inQuietModeNow=true;

  clearDisplay();
  display.setFont(MENUFONT);
  display.setCursor(0, menuFontHeight);
  display.println(F("Be quiet...\nI'm sleeping."));
  if (persistentData.quietTimeEnabled)
  {
    display.println(F("I will wake up at:"));
    displayDate(F(""), persistentData.quietTimeEnd/60, persistentData.quietTimeEnd%60, 0);
  }
  else
    display.println(F("Push button\nto wake up."));
  
  display.dim(true);
  display.display(); 
  strip.setBrightness(neoPixelQuietBrightness); 
  strip.show();
}


void endQuietTimeMode()
{ 
  inQuietModeNow=false;
  display.dim(false);
 // strip.setBrightness(neoPixelNormalBrightness); 
 strip.setBrightness(Brightness);
  strip.show();
  ballDeliveryError=false;
  ballDetectISRCount=0;  //If power was lost during quiet time, this could have been triggered on power on causing a false ball detect

  setMasterTimeFromRtcAndSyncIfNeeded();
}
