#define FASTREAD

//For fast button reads
//Only works with ATmega8/168/328-based board 
//http://masteringarduino.blogspot.com/2013/10/fastest-and-smallest-digitalread-and.html
#ifdef FASTREAD
  #define portOfPin(P)\
    (((P)>=0&&(P)<8)?&PORTD:(((P)>7&&(P)<14)?&PORTB:&PORTC))
  #define ddrOfPin(P)\
    (((P)>=0&&(P)<8)?&DDRD:(((P)>7&&(P)<14)?&DDRB:&DDRC))
  #define pinOfPin(P)\
    (((P)>=0&&(P)<8)?&PIND:(((P)>7&&(P)<14)?&PINB:&PINC))
  #define pinIndex(P)((uint8_t)(P>13?P-14:P&7))
  #define pinMask(P)((uint8_t)(1<<pinIndex(P)))

  #define pinAsInput(P) *(ddrOfPin(P))&=~pinMask(P)
  #define pinAsInputPullUp(P) *(ddrOfPin(P))&=~pinMask(P);digitalHigh(P)
  #define pinAsOutput(P) *(ddrOfPin(P))|=pinMask(P)
  #define digitalLow(P) *(portOfPin(P))&=~pinMask(P)
  #define digitalHigh(P) *(portOfPin(P))|=pinMask(P)
  #define isHigh(P)((*(pinOfPin(P))& pinMask(P))>0)
  #define isLow(P)((*(pinOfPin(P))& pinMask(P))==0)
  #define digitalState(P)((uint8_t)isHigh(P))
#else
  #define isLow(P) (digitalRead(P)==0)
#endif


//Button debounce/deglitch algorithm from:
//https://hackaday.com/2015/12/10/embed-with-elliot-debounce-your-noisy-buttons-part-ii/
#define MASK   0b11000111

ISR(TIMER1_COMPA_vect)
{
   //ISR for timer1, used to scan buttons and store history (for deboune and deglitch)
   //This is done in an ISR instead of loop() so that history is stored at a regular interval
   //loop isn't that regular as it will take much longer if the display is being updated
   //8 bits of history stored, bit is 1 for button press and 0 for button not pressed.

   button1History = button1History << 1;
   button1History |= (isLow(button1Pin));
   button2History = button2History << 1;
   button2History |= (isLow(button2Pin));
  
   
}


uint8_t isButtonPressed(uint8_t *button_history)
{
  uint8_t pressed = 0;    
  if ((*button_history & MASK) == 0b00000111)
  { 
    pressed = 1;
    *button_history = 0b11111111;
  }
  return pressed;
}

uint8_t isButtonReleased(uint8_t *button_history)
{
  uint8_t released = 0;   
  if ((*button_history & MASK) == 0b11000000)
  { 
    released = 1;
    *button_history = 0b00000000;
  }
  return released;
}

uint8_t isButtonDown(uint8_t *button_history)
{
  return (*button_history == 0b11111111);
}

uint8_t isButtonUp(uint8_t *button_history)
{
  return (*button_history == 0b00000000);
}
